/*  Termcap - a library for managing a database of terminal capabilities.
    Copyright (C) 1991 Norman H. Azadian
    Updated in 2001 by Mark Elbrecht.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <termcap.h>
#include <stdio.h>

static char buf[2048];		/* holds termcap entry */
static char strbuf[512];	/* for output of tgetstr() */
static char *strptr;		/* ptr to strbuf[] */


/* Tiny test routine to simulate putting out a character.  */
static int oc(int c)
{
  return putc(c, stdout);
}						/* oc() */


/* Test program for the termcap routines.
   Command line parameters:
  	1st is terminal name, defaulted to "mono".
  	2nd is name of numeric capability, default "co".
  	3rd is name of boolean capability, default "bs".
  	4th is name of string capability, default "so".
  	5th is test string for tgoto(), default is "6\\E&%r%2c%2Y".
  	6th is test string for tputs(), default is "3.5*123".  */
int main(int ac, char **av)
{
  int  ret_code;			/* integer return value */
  char *instr;			/* input string value */
  char *outstr;		/* string return value */
  char *ttype;			/* terminal type string */
  char *capability;		/* capability name string */

  /* setup */
  ospeed = 13;
  PC = '@';
  BC = "H";
  UP = "B";
  
  /* test tgetent() */
  ttype = (ac < 2) ? "mono" : av[1];
  ret_code = tgetent(buf, ttype);
  printf("main: tgetent(buf, \"%s\") returned %d\n", ttype, ret_code);
  if (ret_code != 1)
    exit (99);

  /* test tgetnum() */
  capability = (ac < 3) ? "co" : av[2];
  ret_code = tgetnum(capability);
  printf("main: tgetnum(%s) returned %d\n", capability, ret_code);

  /* test tgetflag() */
  capability = (ac < 4) ? "bs" : av[3];
  ret_code = tgetflag(capability);
  printf("main: tgetflag(%s) returned %d\n", capability, ret_code);

  /* test tgetstr() */
  capability = (ac < 5) ? "so" : av[4];
  strptr = strbuf;
  outstr = tgetstr(capability, &strptr);
  printf("main: tgetstr(%s, 0x%lx) returned '%s'  [strbuf=0x%lx, strptr=0x%lx]\n",
         capability, &strptr, outstr, strbuf, strptr);
  if (strcmp(capability, "so") == 0)
  {
    strptr = strbuf;
    tgetstr("se", &strptr);
    printf(strbuf);
  }

  /* test tgoto() */
  instr = (ac < 6) ? "6\\E&%r%2c%2Y" : av[5];
  outstr = tgoto(instr, 12, 3);
  printf("main: tgoto(\"%s\", 12, 3) returned '%s'\n", instr, outstr);

  /* test tputs() */
  instr = (ac < 7) ? "3.5*123" : av[6];
  printf("main: tputs(\"%s\", 3, oc) returned '", instr);
  tputs(instr, 3, oc);
  printf("'\n");

  return 0;
}
