/*  Termcap - a library for managing a database of terminal capabilities.
    Copyright (C) 1991 Norman H. Azadian
    Updated in 2001 by Mark Elbrecht.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <termcap.h>
#include <stdio.h>

char *BC;
char *UP;

/* Returns a cursor addressing string decoded from cm to go to column destcol
   in line destline. It uses the external variables UP (from the up capability)
   and BC (if bc is given rather than bs) if necessary to avoid placing
   \n, ^D, or ^@ in the returned string.  (Programs which call tgoto() should
   be sure to turn off the XTABS bit(s), since tgoto() may not output a tab.
   Note that programs using termcap should in general turn off XTABS anyway
   since some terminals use control I for other functions, such as non-
   destructive space.)  If a % sequence is given which is not understood,
   then tgoto() returns "OOPS".
   The output buffer is local, so don't try any recursion.
   No error checking here.  */
char *tgoto(char *cm, int destcol, int destline)
{
  char *outp;			/* pointer into answer[] */
  static char answer[88];	/* result stashed here */
  int reversed_flag;		/* should send col 1st ? */
  int value;			/* next value to output */

  reversed_flag = 0;
  value = destline;
  outp = answer;
  for ( ; *cm; ++cm)
  {
    if (*cm == '%')
    {
      switch (*++cm)
      {
        case '%':
          *outp++ = '%';
          break;

        case 'd':
          sprintf(outp, "%d", value);
          if (value < 0)
            ++outp;
          ++outp;
          if (abs(value) > 9)
            ++outp;
          if (abs(value) > 99)
            ++outp;
          value = (reversed_flag) ? destline : destcol;
          break;

        case '2':
          sprintf(outp, "%02d", value);
          outp += 2;
          value = (reversed_flag) ? destline : destcol;
          break;

        case '3':
          sprintf(outp, "%03d", value);
          outp += 3;
          value = (reversed_flag) ? destline : destcol;
          break;

        case '.':
          *outp++ = value;
          value = (reversed_flag) ? destline : destcol;
          break;

        case '+':
          *outp++ = value + *++cm;
          value = (reversed_flag) ? destline : destcol;
          break;

        case '>':
          if (value > *++cm)
            value += *++cm;
          else
            ++cm;
          break;

        case 'r':
          value = (reversed_flag) ? destline : destcol;
          reversed_flag ^= 1;
          break;

        case 'i':
          ++value;
          ++destline;
          ++destcol;
          break;

        case 'n':
          destcol ^= 0140;
          destline ^= 0140;
          break;

        case 'B':
          value = (16 * (value / 10)) + (value % 10);
          break;

        case 'D':
          value = (value - (2 * (value % 16)));
          break;

        default:
          sprintf(outp, "OOPS");
          outp += 4;
          break;
      }
    }
    else
      *outp++ = *cm;
  }
  *outp = '\0';
  return answer;
}

