/*  Termcap - a library for managing a database of terminal capabilities.
    Copyright (C) 1991 Norman H. Azadian
    Updated in 2001 by Mark Elbrecht.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <termcap.h>
#include <stdlib.h>

/* Output speed of terminal as encoded by stty.  */
short ospeed;

/* Pad character (from the pc capability) if a null character
   is inappropriate.  */
char PC;

/* Milliseconds per character, for each of the possible baud rates
   of ospeed.  Here multiplied by 10 for computational convenience.  */
static unsigned	delay_factor[] =
{
     0,	/* B0 */	/* hang up dataphone */
  1920,	/* B50 */
  1280,	/* B75 */
   872,	/* B110 */
   730,	/* B134 */
   640,	/* B150 */
   480,	/* B200 */
   320,	/* B300 */
   160,	/* B600 */
    80,	/* B1200 */
    50,	/* B1800 */
    40,	/* B2400 */
    20,	/* B4800 */
    10,	/* B9600 */
     5,	/* EXTA (19200 here) */
     2,	/* EXTB (34800 here) */
};

void tputs(char *cp, int affcnt, int (*outc)(int))
{
  char *p;
  int decimal_flag;             /* delay had a decimal point */
  unsigned delay;
  unsigned cnt;

  if (cp == NULL || *cp == '\0')
  {
    return;
  }

  /* Calculate delay, if any.  */
  decimal_flag = 0;
  delay = 0;
  while (isdigit(*cp))
  {
    delay = delay * 10 + (*cp - '0');
    ++cp;
  }
  if (*cp == '.')
  {
    ++cp;
    if (isdigit(*cp))
    {
      delay += *cp - '0';
      ++cp;
    }
    /* Only one digit permitted to the right of the decimal point.  */
    while (isdigit(*cp))
      ++cp;
  }
  if (*cp == '*')
  {
    delay *= affcnt;
    ++cp;
  }

  /* Output string.  */
  for ( ; *cp != '\0'; ++cp)
    outc(*cp);

  /* If no delay, return now.  */
  if (delay == 0 || ospeed <= 0 ||
      ospeed > (sizeof(delay_factor) / sizeof(delay_factor[0])))
  {
    delay += (delay_factor[ospeed] + 1) / 2;	/* round up */
    if (delay)
      delay /= delay_factor[ospeed];
  }

  if (delay == 0)
    return;

  /* Output delay characters.  */
  for (cnt = delay; cnt; --cnt)
    outc(PC);
}
